@@ -11,6 +11,7 @@ override _build_WriteMakefile_args => sub {
     my $self = shift;
 
     my $args = super();
+
     # This makes it build the perl_math_int64.c file into a .o and then link
     # it with Int128.o
     $args->{OBJECT} = '$(O_FILES)';
@@ -26,7 +27,8 @@ override _build_WriteMakefile_dump => sub {
 
     my $dump = super();
     $dump .= <<'EOF';
-$WriteMakefileArgs{DEFINE} = _int128_define();
+$WriteMakefileArgs{DEFINE}  = _int128_define();
+$WriteMakefileArgs{CCFLAGS} = _ccflags( $WriteMakefileArgs{CCFLAGS} );
 EOF
 
     return $dump;
@@ -48,6 +50,8 @@ __PACKAGE__->meta()->make_immutable();
 
 __DATA__
 
+use Config qw( %Config );
+
 use lib 'inc';
 use Config::AutoConf;
 
@@ -57,7 +61,7 @@ sub _check_for_capi_maker {
     unless ( eval { require Module::CAPIMaker; 1; } ) {
         warn <<'EOF';
 
-  It looks like you're trying to build Math::Int64 from the git repo. You'll
+  It looks like you're trying to build Math::Int128 from the git repo. You'll
   need to install Module::CAPIMaker from CPAN in order to do this.
 
 EOF
@@ -89,8 +93,8 @@ EOF
 # i386 platforms. See http://llvm.org/bugs/show_bug.cgi?id=15834 for the bug
 # report. This appears to be
 sub _check_type {
-    my $autoconf = shift;
-    my $type     = shift;
+    my $autoconf     = shift;
+    my $uint128_type = shift;
 
     my $uint64_type
         = $autoconf->check_type('uint64_t') ? 'uint64_t'
@@ -99,27 +103,44 @@ sub _check_type {
         ? 'unsigned int __attribute__ ((__mode__ (DI)))'
         : return 0;
 
-    my $cache_name = $autoconf->_cache_type_name( 'type', $type );
+    my $cache_name = $autoconf->_cache_type_name( 'type', $uint128_type );
     my $check_sub = sub {
         my $prologue = $autoconf->_default_includes();
-        $prologue .=
-            $type =~ /__mode__/
-            ? "typedef unsigned uint128_t __attribute__((__mode__(TI)));\n"
-            : "typedef unsigned __int128 uint128_t;\n";
 
-        # The rand() calls are there because if we just use constants than the
-        # compiler can optimize most of this code away.
         my $body = <<"EOF";
-$uint64_type a = (($uint64_type)rand()) * rand();
-$uint64_type b = (($uint64_type)rand()) << 24;
-uint128_t c = ((uint128_t)a) * b;
-return c > rand();
+volatile $uint64_type a = (($uint64_type)3) * 4;
+volatile $uint64_type b = (($uint64_type)5) << 24;
+volatile $uint128_type c = (($uint128_type)a) * b;
+return c > 0;
 EOF
+
         my $conftest = $autoconf->lang_build_program( $prologue, $body );
         return $autoconf->compile_if_else($conftest);
     };
 
-    return $autoconf->check_cached( $cache_name, "for $type", $check_sub );
+    return $autoconf->check_cached(
+        $cache_name,
+        "for $uint128_type",
+        $check_sub
+    );
+}
+
+sub _ccflags {
+    my $flags = shift;
+
+    my $config_flags = $Config{ccflags};
+    if ($config_flags) {
+        $config_flags =~ s/ ?-arch i386//
+            if $config_flags =~ /-arch x86_64/
+            && $config_flags =~ /-arch i386/;
+    }
+
+    $flags = join q{ }, grep { defined && length } $flags, $config_flags;
+
+    return $flags unless -d '.git';
+
+    return join q{ }, ( $flags || q{} ),
+        qw( -Wall -Wdeclaration-after-statement );
 }
 
 package MY;
